# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class TemplateMV2levwishRSIndex(Template):
    'Fits 2-level multivariate Normal response models, allowing for random slopes, missing data, and a choice of uniform or Wishart priors.'

    __version__ = '1.0.0'

    tags = [ 'Model', '2-Level', 'Normal', 'Multivariate response', 'Random slopes', 'MLwiN:point & click']
    engines = ['eStat', 'MLwiN_MCMC', 'R_MCMCglmm']

    inputs = '''
y = DataMatrix('Responses: ')
L2ID = IDVector('Level 2 ID: ')
for i in range(0, len(y)):
    context['xf'+str(i+1)] = DataMatrix('Explanatory variables for response ' + y[i] + ': ', allow_cat = True)
    context['xr'+str(i+1)] = DataMatrix('Explanatory variables random for response ' + y[i]+ '?: ', allow_cat = True)

storeresid = Boolean('Store residuals?')

lenbeta = 0
lenu = 0
for i in range(0, len(y)):
    lenbeta += len(context['xf'+str(i+1)])
    lenu += len(context['xr'+str(i+1)])

for i in range(0, len(y)):
    context['miss'+y[i]] = ParamVector(monitor=False)
    for x in range(0, len(context['xr'+str(i+1)])):
        if storeresid:
            context['u' + str(x) + '_' + str(i)] = ParamVector(parents=[L2ID], as_scalar=False)
        else:
            context['u' + str(x) + '_' + str(i)] = ParamVector(parents=[L2ID], as_scalar=False, monitor=False)

n = len(y)
if n == 1:
    tau = ParamScalar()
    sigma = ParamScalar(modelled = False)
    tau_u = ParamScalar()
    sigma_u = ParamScalar(modelled = False)
else:
    omega_e = ParamMatrix(modelled = False, customstep=True)
    omega_e.size = n
    omega_u = ParamMatrix(modelled = False, customstep=True)
    omega_u.size = lenu
    d_e = ParamMatrix(customstep=True)
    d_e.size = n
    d_u = ParamMatrix(customstep=True)
    d_u.size = lenu
    priors = Text('Priors: ', ['Uniform', 'Wishart'])
    if priors == 'Wishart':
        R = List('R matrix: ')
        v = Integer('Degrees of Freedom:')
    priors2 = Text('Level 2 priors: ', ['Uniform', 'Wishart'])
    if priors2 == 'Wishart':
        R2 = List('R matrix: ')
        v2 = Integer('Degrees of Freedom:')

beta = ParamVector()
beta.ncols = lenbeta
deviance = ParamScalar(modelled=False) 
'''

    model = '''
<% lenbeta = 0 %>
<% lenu = 0 %>
<% n = len(y) %>
model {
    for (i in 1:length(${y[0]})) {
    % if n == 1:
            miss${y[0]}[i] ~ dnorm(mu0[i], tau)
            mu0[i] <- ${mmult2(context['xr1'], 'beta', 'i', lenbeta)} + u0[${L2ID}[i]]
<% lenbeta += len(context['xf1']) %> \\
    % else:
            dummy[i] ~ dnormal${n}a( \\
        % for i in range(0,n):
miss${y[i]}[i], \\
        % endfor
        % for i in range(0,n):
mu${i}[i], \\
        % endfor
        % for i in range(0,(((n+1)*n)/2) - 1):
d_e[${i}], \\
        % endfor
d_e[${(((n+1)*n)/2) - 1}])
        % for i in range(0, len(y)):
            mu${i}[i] <- ${mmult2(context['xf' + str(i+1)], 'beta', 'i', lenbeta)} \\

            % for j in range(0, len(context['xr'+str(i+1)])):
            + u${j}_${i}[${L2ID}[i]] * ${context['xr'+str(i+1)][j]}[i]
            % endfor
<% lenbeta += len(context['xf' + str(i+1)]) %> \\
<% lenu += len(context['xr' + str(i+1)]) %> \\
        % endfor
        % for i in range(0,n):
        miss${y[i]}[i] ~ dflat()
        % endfor
    % endif
    }
      
    for (j in 1:length(u0_0)) {
        %if len(y) == 1 :
           u0[j] ~ dnorm(0, tau_u)
    % else:
        dummy_u[j] ~ dnormal${lenu}a( \\
% for i in range(0, len(y)):
 % for j in range(0, len(context['xr'+str(i+1)])):
u${j}_${i}[j], \\
        % endfor
        % endfor
        % for i in range(0,lenu):
0, \\
        % endfor
        % for i in range(0,(((lenu+1)*lenu)/2) - 1):
d_u[${i}], \\
        % endfor
d_u[${(((lenu+1)*lenu)/2) - 1}])

        dum_u[j] ~ ddummy(dummy_u[j])
% for i in range(0, len(y)):
 % for j in range(0, len(context['xr'+str(i+1)])):
        u${j}_${i}[j] ~ dflat()
        % endfor
        % endfor

    % endif

    }

    # Priors
    
    % if n == 1:
    tau ~ dgamma(0.001, 0.001)
    sigma <- 1 / sqrt(tau)
    tau_u ~ dgamma(0.001, 0.001)
    sigma_u <- 1 / sqrt(tau_u)
    % endif

    % for i in range(0, lenbeta):
    beta_${i} ~ dflat()
    % endfor   
}       
''' 

    latex = r'''
<% lenbeta = 0 %>
<% n = len(y) %>
\begin{aligned} 
\left(
\begin{array}{l}
% for r in range(0, len(y)):
\mbox{${y[r]}}_i 
% if r != len(y) -1:
\\ 
% endif
% endfor
\end{array}
\right) & \sim \mbox{N}
\left[ \left(
\begin{array}{l}
% for r in range(0, len(y)):
\mu_{${r}i}
% if r != len(y) -1:
\\ 
% endif
% endfor
\end{array}
\right), \Omega_e \right] \\ 
% for r in range(0, len(y)):
\mu_{${r}i} & = ${mmulttex2(context['xf' + str(r+1)], r'\beta', 'i', lenbeta)}
% for j in range(0, len(context['xr' + str(r + 1)])):
+ u_{${r},${j},${L2ID}[i]}^{(2)} \mbox{${context['xr' + str(r + 1)][j]}}_{i}
% endfor
\\ 
<% lenbeta += len(context['xf' + str(r+1)]) %>
% endfor

\left(
\begin{array}{l}
% for r in range(0, len(y)):
% for j in range(0, len(context['xr' + str(r + 1)])):
u^{(2)}_{${r}, ${j},${L2ID}(i)} 
% if j != len(context['xr' + str(r + 1)]) - 1 or r != len(y) - 1:
\\ 
% endif
% endfor
% endfor
\end{array}
\right) & \sim \mbox{N}

\left[ \left(
\begin{array}{l}
% for r in range(0, len(y)):
% for i in range(0, len(context['xr' + str(r + 1)])):
0
% if i != len(context['xr' + str(r + 1)]) -1 or r != len(y) - 1:
\\ 
% endif
% endfor
% endfor
\end{array}
\right), \Omega^{(2)}_{u} \right] \\ 


<% lenu = 0 %>
% for i in range(0, len(y)):
<% lenu += len(context['xr' + str(i+1)]) %>
% endfor

% if priors2 == 'Wishart':
\Omega^{(2)}_u & \sim \mbox{inverse Wishart}_{${lenu}}[${v2}*R_{u2},${v2}],R_{u2}=
\left[\begin{array}{
% for i in range(0, lenu):
l
% endfor
}
<% lenR = 0%>
% for i in range(0, lenu):
% for j in range(0, lenu):
% if j <= i:
${R2.name[lenR]}
<% lenR += 1 %>
% endif
% if j != lenu - 1:
&
% else:
% if i != lenu - 1:
\\ 
% endif
% endif
% endfor
% endfor
\end{array}
\right]
% endif
% if priors2 == 'Uniform':
\Omega^{(2)}_u & \propto 1
% endif
\\ 


% if priors == 'Wishart':
\Omega_e & \sim \mbox{inverse Wishart}_{${len(y)}}[${v}*R_e,${v}],R_e=
\left[\begin{array}{
% for i in range(0, len(y)):
l
% endfor
}
<% lenR = 0%>
% for i in range(0, len(y)):
% for j in range(0, len(y)):
% if j <= i:
${R.name[lenR]}
<% lenR += 1 %>
% endif
% if j != len(y) - 1:
&
% else:
% if i != len(y) - 1:
\\ 
% endif
% endif
% endfor
% endfor
\end{array}
\right]
% endif
% if priors == 'Uniform':
\Omega_e & \propto 1
% endif
\\ 

% for i in range(0, lenbeta):
\beta_${i} & \propto 1 \\ 
% endfor   

\end{aligned} 
'''

    preccode = '''
<%!
    def mmult2(names, var, index,count):
        out = ""
        first = True
        for name in names:
            if first == False:
                out += ' + '
            else:
                first = False
            out += 'double(' + name + '[' + index + ']) * ' + var + '_' + str(count)
            count += 1
        return out
%>
    {
<% n = len(y) %>\\
        // Note currently using a uniform prior for variance matrix
        SymMatrix sbe(${n});
        int vwe = length(${y[0]});
        for(int i = 0; i < length(${y[0]}); i++) {
<% lenbeta = 0 %>\\
% for i in range(0, n):
            double e${i} = double(miss${y[i]}[i]) - (${mmult2(context['xf' + str(i+1)], 'beta', 'i', lenbeta)}) \\
% for j in range(0, len(context['xr'+str(i+1)])):
 - double(u${j}_${i}[int(${L2ID}[i])]) * double(${context['xr' + str(i+1)][j]}[i])\\
%endfor
;\\
<% lenbeta += len(context['xf' + str(i + 1)]) %>
% endfor

% for i in range(0, n):
% for j in range(i, n):
            sbe(${i}, ${j}) += e${i} * e${j};
% endfor
% endfor
        }
% if priors == 'Uniform':
        if (runstate == 0) {
% for i in range(0, n):
            sbe(${i}, ${i}) += 0.0001;
% endfor
        }
% endif
% if priors == 'Wishart':
<%
import numpy
Rmat = numpy.empty([n, n])
count = 0 
for i in range(0, n):
    for j in range(0, i + 1):
        Rmat[i, j] = float(R.name[count]) 
        Rmat[j, i] = Rmat[i, j]
        count += 1
%>

<% count = 0 %>
% for i in range(0, n):
% for j in range(i, n):
        sbe(${i}, ${j}) += ${str(Rmat[i][j] * float(v))};
% endfor
% endfor
% endif

        matrix_sym_invinplace(sbe);

% if priors == 'Uniform':
        vwe -= ${n + 1};
% endif
% if priors == 'Wishart':
        vwe += ${v};
% endif

        bool fail = false;
        mat_d_e = dwishart(vwe, sbe, fail);
        mat_omega_e = matrix_sym_inverse(mat_d_e, fail);

<% lenu = 0 %>\\
% for i in range(0, len(y)):
<% lenu += len(context['xr' + str(i+1)]) %>\\
% endfor
        SymMatrix sbu(${lenu});
        for(int i=0;i<length(u0_0);i++) {
<% loop1 = 0 %>\\
% for i1 in range(0, len(y)):
% for j1 in range(0, len(context['xr'+str(i1+1)])):
<% loop2 = 0 %>\\
% for i2 in range(0, len(y)):
% for j2 in range(0, len(context['xr'+str(i2+1)])):
% if loop2 >= loop1:
            sbu(${loop1}, ${loop2}) += double(u${j1}_${i1}[i]) * double(u${j2}_${i2}[i]);
% endif
<% loop2 += 1 %>\\
% endfor
% endfor
<% loop1 += 1 %>\\
% endfor
% endfor
        }
% if priors2 == 'Uniform':
        if (runstate == 0) {
% for i in range(0, loop1):
            sbu(${i}, ${i}) += 0.0001;
% endfor
        }
% endif
% if priors2 == 'Wishart':
<%
import numpy
Rmat = numpy.empty([loop1, loop1])
count = 0 
for i in range(0, loop1):
    for j in range(0, i + 1):
        Rmat[i, j] = float(R2.name[count]) 
        Rmat[j, i] = Rmat[i, j]
        count += 1
%>

<% count = 0 %>
% for i in range(0, loop1):
% for j in range(i, loop1):
        sbu(${i}, ${j}) += ${str(Rmat[i, j] * float(v2))};
% endfor
% endfor
% endif
% if priors2 == 'Uniform':
        int vwu = length(u0_0) - ${loop1 + 1};
% endif
% if priors2 == 'Wishart':
        int vwu = length(u0_0) + ${v2};
% endif

        matrix_sym_invinplace(sbu);
        mat_d_u = dwishart(vwu, sbu, fail); 
        mat_omega_u = matrix_sym_inverse(mat_d_u, fail);
    }
         '''

    mlnscript = '''
<% M=len(y) %>

% for i in range(len(y)):
MVAR 1 "${y[i]}"
% endfor
IDEN 3 "${L2ID}"
IDEN 2 "_id"
IDEN 1 "resp_indicator"

<%
import numpy
T=numpy.eye(len(y), dtype='int')
%>
% for j in range(0, len(T)):
RPAT ${' '.join([str(k) for k in T[j]])}
% for i in range(0,len(context['xf'+str(j+1)])):
ADDT "${context['xf'+str(j+1)][i]}"
% endfor
ADDT "_levres"
RPAT
% endfor

% for i in range(len(y)):
% for j in range(0,len(context['xr'+str(i+1)])):
SETV 3 "${context['xr'+str(i+1)][j]}.${str(i+1)}"
% endfor
SETV 2 "_levres.${i + 1}"
FPAR 0 "_levres.${i + 1}"
% endfor

<%
import numpy

TT = ' '.join([str(i) for i in numpy.zeros(int(beta.ncols),dtype='int')])

if priors == 'Wishart':
    TR = ' '.join(R.name)

if priors2 == 'Wishart':
    TR2 = ' '.join(R2.name)
%>
% if priors2 == 'Wishart' and priors !='Wishart':
JOIN ${TT} 1 ${TR2} ${str(v2)} 0 0 c1092
PRIOR  c1092 
% endif
% if priors == 'Wishart' and priors2 !='Wishart':
JOIN ${TT} 0 1 ${TR} ${str(v)} 0 c1092
PRIOR  c1092 
% endif
% if priors == 'Wishart' and priors2 =='Wishart':
JOIN ${TT} 1 ${TR2} ${str(v2)} 1 ${TR} ${str(v)} 0 c1092
PRIOR  c1092 
% endif
        '''

    latexMLwiN = r'''
\begin{aligned}
&\text{Select }\bold{Open}\text{ }\bold{Sample}\text{ } \bold{Worksheet} \text{ or }\bold{Open}\text{ } \bold{Worksheet} \text{ from the }\bold{File}\text{ menu.} \\ 
&\text{Select }\bold{datafile.dta}.\\ 
&\text{Select }\bold{Equations}\text{ from the }\bold{Model}\text{ menu.}\\ 
&\text{Click on the } \bold{Responses}\text{ button at the bottom of the window.}\\ 
% for i in range(len(y)):
&\text{Click on }\bold{${y[i]}}\text{ from the list that appears.}\\ 
% endfor
&\text{Click on the }\bold{Done}\text{ button on the }\bold{Responses} \text{ window.}\\ 
&\text{On the }\bold{Equations}\text{ window click on the response names to bring up the }\mathbf{Y}\text{ }\bold{variable}\text{ window}.\\ 
&\text{On the }\mathbf{Y}\text{ }\bold{variable}\text{ window change the number of levels to } \mathbf{3}-\mathbf{ijk}.\\ 
&\text{In the }\bold{level} \text{ }\mathbf{3(k):}\text{ list, select } \bold{${L2ID}}.\\  
&\text{In the }\bold{level} \text{ }\mathbf{2(j):}\text{ list, select } \bold{\_id}.\\  
&\text{Click on the }\bold{Done}\text{ button.}\\ 
&\text{Click the }\bold{Add}\text{ }\bold{Term}\text{ button on the tool bar.}\\ 
&\text{Select }\bold{\_levres}\text{ from the variable list.}\\ 
&\text{Click on the }\bold{Add}\text{ }\bold{Separate}\text{ }\bold{coefficients}\text{ button.}\\

<%
tt=list()
for i in range(0, len(y)):
    for j in range(len(context['xf'+str(i+1)])):
        tt.append( context['xf'+str(i+1)].name[j])

tt=list(set(tt))
%>
% for i in range(len(tt)):
&\text{Click the }\bold{Add}\text{ }\bold{Term}\text{ button on the tool bar.}\\ 
&\text{Select }\bold{${tt[i]}}\text{ from the variable list.}\\ 
&\text{Click on the }\bold{Add}\text{ }\bold{Separate}\text{ }\bold{coefficients}\text{ button.}\\ 
% endfor
%for i in range(0, len(y)):
%for j in range(len(tt)):
%if tt[j] not in context['xf'+str(i+1)].name:
&\text{On the }\bold{Equations}\text{ window click on }\bold{${tt[j]}.${y[i]}}.\\ 
&\text{Remove the tick in the }\bold{Fixed}\text{ }\bold{parameter}\text{ tick box.}\\ 
&\text{Click on the }\bold{Done}\text{ button.}\\ 
%endif
%endfor
%endfor

% for i in range(len(y)):
% for j in range(0,len(context['xr'+str(i+1)])):
&\text{On the }\bold{Equations}\text{ window click on }\bold{${context['xr'+str(i+1)][j]}.${y[i]}}.\\ 
&\text{Add the tick in the }\mathbf{k}(\bold{${L2ID}\_long})\text{ tick box.}\\ 
&\text{Click on the }\bold{Done}\text{ button.}\\ 
% endfor
&\text{On the }\bold{Equations}\text{ window click on }\bold{\_levres.${y[i]}}.\\ 
&\text{Remove the tick in the }\bold{Fixed}\text{ }\bold{parameter}\text{ tick box.}\\ 
&\text{Add the tick in the }\mathbf{j}(\bold{\_id\_long})\text{ tick box.}\\ 
&\text{Click on the }\bold{Done}\text{ button.}\\ 
% endfor

'''

    latexMLwiNextra = r'''
&\text{Select the }\bold{MCMC}\text{ tab from the }\bold{Estimation}\text{ }\bold{control}\text{ window.}\\ 
&\text{Select }\bold{MCMC}/\bold{Priors}\text{ from the }\bold{Model}\text{ menu.}\\ 
&\text{Select }\bold{Gamma}\text{ }\bold{prior}\text{ from the } \bold{MCMC}\text{ }\bold{Priors}\text{ window (this sets the prior for level 1 variance).}\\ 
% if priors2 == 'Wishart':
&\text{Select }\bold{Informative}\text{ }\bold{prior} \text{ from the } \bold{MCMC}\text{ }\bold{Priors}\text{ window.}\\ 
&\text{Select }\Sigma_v \text{ from the top bar of the window.}\\ 
&\text{Input the estimates }
%for i in range(len(R2.name)):
%if i==(len(R2.name)-1):
\text{and }\mathbf{${R2.name[i]}}
%else:
\mathbf{${R2.name[i]}}\text{, }
%endif
%endfor
\text{ in the estimate boxes.}\\ 
&\text{Input } \mathbf{${v2.name}} \text{ in the sample size box.}\\ 
&\text{Click on }\Sigma_v \text{ again.}\\ 
%endif
% if priors == 'Wishart':
&\text{Select }\Sigma_u \text{ from the top bar of the window.}\\ 
&\text{Input the estimates }
%for i in range(len(R.name)):
%if i==(len(R.name)-1):
\text{and }\mathbf{${R.name[i]}}
%else:
\mathbf{${R.name[i]}}\text{, }
%endif
%endfor
\text{ in the estimate boxes.}\\ 
&\text{Input } \mathbf{${v.name}} \text{ in the sample size box.}\\ 
&\text{Click on }\Sigma_u \text{ again.}\\ 
%endif
&\text{Click on the }\bold{Done}\text{ button.}\\ 
'''

    rscript = '''
<%
import re
from EStat.Templating import Rmmult

data = {}
for var in datafile.variables.keys():
    data[var] = datafile.variables[var]['data']

N=datafile.nobs
M=len(context['y'])

resp='cbind('
nparams=0
for i in range(M):
    resp+=str(y[i])
    nparams+=len(context['xf'+str(i+1)])
    if i!= M-1:
        resp+=','
    else:
        resp+=')'
TT=1
for i in range(M-1,0,-1):
    if len(context['xf'+str(i+1)])!=len(context['xf'+str(i)]):
        TT=0
    else:
        aa=context['xf'+str(i+1)].name
        aa.sort()
        bb=context['xf'+str(i)].name
        bb.sort()
        for j in range(len(aa)):
            if aa[j]!=bb[j]:
                TT=0
SS=1
for i in range(M-1,0,-1):
    if len(context['xr'+str(i+1)])!=len(context['xr'+str(i)]):
        SS=0
    else:
        aa=context['xr'+str(i+1)].name
        aa.sort()
        bb=context['xr'+str(i)].name
        bb.sort()
        for j in range(len(aa)):
            if aa[j]!=bb[j]:
                SS=0
                
lenu=len(xr1)
    
%>

% if Rpackage == 'MCMCglmm':
% if D == 'MVNormal':
% if TT!=0 and SS!=0:
formula <- ${resp} ~ trait:(${Rmmult(xf1)}) - 1
##define the prior...use non-informative prior #fixed effect N(0,10^6), variance~IW(1,0.002)~IG(0.001000,0.001000)
#random effect Uniform: G1~IW(vw2,diag(M)); Wishart: Rmat2 
#Residual Uniform: G1~IW(vw,diag(M)); Wishart: Rmat
% if priors2.name == 'Uniform':
vw2 = ${M*lenu + 1};
% endif
% if priors2.name == 'Wishart':
vw2 =  ${v2.name};
Rmat2 = matrix(0,${M*lenu},${M*lenu})
Rvec2=rep(0,${len(R2.name)})
%for i in range(0,len(R2.name)):
Rvec2[${i+1}]=as.numeric(${float(R2.name[i])})
%endfor
count = 0 
for (j in  0:${M*lenu-1}){
    for (k in 0:j ){
        Rmat2[j+1, k+1] = as.numeric(Rvec2[count+1])
        Rmat2[k+1, j+1] = Rmat2[j+1, k+1]
        count = count+ 1
    }
}
% endif

% if priors.name == 'Uniform':
vw = ${M + 1};
% endif
% if priors.name == 'Wishart':
vw = ${v.name};
Rmat = matrix(0,${M},${M})
Rvec=rep(0,${len(R.name)})
%for i in range(0,len(R.name)):
Rvec[${i+1}]=as.numeric(${float(R.name[i])})
%endfor
count = 0 
for (j in  0:${M-1}){
    for (k in 0:j ){
        Rmat[j+1, k+1] = as.numeric(Rvec[count+1])
        Rmat[k+1, j+1] = Rmat[j+1, k+1]
        count = count+ 1
    }
}
% endif

% if priors.name == 'Uniform' and priors2.name == 'Uniform':
myprior<-list(B=list(V=diag(${nparams})*10^6,mu=rep(0,${nparams})),G=list(G1=list(V=diag(${M*lenu}),nu=vw2)),R=list(V=diag(${M}),nu=vw))
% endif
% if priors.name == 'Wishart' and priors2.name == 'Uniform':
myprior<-list(B=list(V=diag(${nparams})*10^6,mu=rep(0,${nparams})),G=list(G1=list(V=diag(${M*lenu}),nu=vw2)),R=list(V=Rmat,nu=vw))
% endif
% if priors.name == 'Uniform' and priors2.name == 'Wishart':
myprior<-list(B=list(V=diag(${nparams})*10^6,mu=rep(0,${nparams})),G=list(G1=list(V=Rmat2,nu=vw2)),R=list(V=diag(${M}),nu=vw))
% endif
% if priors.name == 'Wishart' and priors2.name == 'Wishart':
myprior<-list(B=list(V=diag(${nparams})*10^6,mu=rep(0,${nparams})),G=list(G1=list(V=Rmat2,nu=vw2)),R=list(V=Rmat,nu=vw))
% endif





myModel<-MCMCglmm(formula,random=~us(trait:(
%for j in range(len(xr1)):
${xr1[j]}
%if j!=len(xr1)-1:
+
%else:
)):${L2ID}
%endif
%endfor
, rcov=~us(trait):units,family=rep("gaussian",${M}),prior=myprior,burnin=${burnin},nitt=${iterations + burnin},thin=${thinning},data=mydata,verbose=FALSE)

##summary and outputs from the model
## combined output fixed and variance into one matrix 

summary(myModel)


myModelOut<-cbind(myModel$Sol,myModel$VCV)
myModelOut<-mcmc(myModelOut)
summary(myModelOut)

##temporary plots
<% ind=divmod(nparams,3) %>
% for i in range(ind[0]):
<% svgout='DiagPlots'+str(i+1)+'.svg' %>
svg("${svgout}")
plot(myModelOut[,(1+3*${i}):(3*${i+1})])
dev.off()
% endfor
% if (ind[1]>0):
<% svgout='DiagPlots'+str(ind[0]+1)+'.svg' %>
svg("${svgout}")
plot(myModelOut[,(1+3*${ind[0]}):${nparams}])
dev.off()
% endif
## export chains
colnames(myModelOut)[which(colnames(myModelOut)=="units")]<-"sigma^2"
write.dta(data.frame(myModelOut),file="chains.dta")
## export DIC
dic <- myModel$DIC
write.dta(data.frame(dic), file = "dic.dta")


%else:
<%print 'The model not implemented yet, choose another engine:'%>
%endif
% else:
<% print 'Distribution not implemented yet, choose another engine:' %>
% endif
% endif
'''
   
    preparedata = '''
mydata = data['datafile']

for i in range(0, len(y)):
    context['miss'+y[i]].ncols = -1*len(mydata.variables[y[i]]['data'])
'''
