# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class TemplateNLevMVNormCC(Template):
    'Fits multivariate Normal response models with one or more higher classifications (nested or crossed), allowing for missing data, and a choice of uniform or Wishart priors. Creates custom C code.'

    __version__ = '1.0.0'

    tags = [ 'Model', 'N-Level', 'Normal', 'Multivariate response' ]
    engines = ['CustomC']

    inputs = '''
NumLevs = Integer('Number of Classifications: ')
for i in range(0, int(NumLevs)):
    selstr = 'Classification ' + str(i + 1) + ': '
    context['C' + str(i + 1)] = IDVector(selstr)
y = DataMatrix('Responses: ')

for i in range(0, len(y)):
    context['x'+str(i+1)] = DataMatrix('Explanatory variables for response ' +y[i] + ': ', allow_cat = True)

storeresid = Boolean('Store residuals?')

lenbeta = 0
for i in range(0, len(y)):
    lenbeta += len(context['x'+str(i+1)])
    context['miss' + y[i]] = ParamVector(monitor=False)
    for j in range(0, int(NumLevs)):
        if storeresid:
            context['u' + str(j) + '_' + str(i)] = ParamVector(parents=[context['C' + str(j + 1)]], as_scalar=False)
        else:
            context['u' + str(j) + '_' + str(i)] = ParamVector(parents=[context['C' + str(j + 1)]], as_scalar=False, monitor=False)

n = len(y)
if n == 1:
    tau = ParamScalar()
    sigma = ParamScalar(modelled = False)
    for i in range(0, int(NumLevs)):
        context['tau_u' + str(i)] = ParamScalar()
        context['sigma_u' + str(i)] = ParamScalar(modelled = False)
else:
    omega_e = ParamMatrix(modelled = False)
    omega_e.size = n
    d_e = ParamMatrix()
    d_e.size = n
    priors = Text('Priors: ', ['Uniform', 'Wishart'])
    if priors == 'Wishart':
        R = List('R matrix: ')
        v = Integer('Degrees of Freedom:')
    for i in range(0, int(NumLevs)):
        context['omega_u' + str(i+1)] = ParamMatrix(modelled = False)
        context['omega_u' + str(i+1)].size = n
        context['d_u' + str(i+1)] = ParamMatrix()
        context['d_u' + str(i+1)].size = n
        context['priors' + str(i)] = Text('Priors at ' + context['C' + str(i + 1)] + ' classification: ', ['Uniform', 'Wishart'])
        if context['priors' + str(i)] == 'Wishart':
            context['R' + str(i)] = List('R matrix: ')
            context['v' + str(i)] = Integer('Degrees of Freedom:')


beta = ParamVector()
beta.ncols = lenbeta

deviance = ParamScalar(modelled = False)
'''

    model = '''
<% lenbeta = 0 %> \\
<% numlevs = int(NumLevs) %> \\
<% n = len(y) %> \\
model {

        for (i in 1:length(${y[0]})) {
    % if n == 1:
            miss0[i]  ~ dnorm(mu[i], tau)
            mu0[i] <- ${mmult2(context['x1'], 'beta', 'i', lenbeta)} \\
% for i in range(0, numlevs):
+ u0_${i}[${context['C' + str(i + 1)]}[i]]
% endfor
<% lenbeta += len(context['x1']) %> \\
    % else:
            dummy[i] ~ dnormal${n}a( \\
        % for i in range(0,n):
miss${y[i]}[i], \\
        % endfor
        % for i in range(0,n):
mu${i}[i], \\
        % endfor
        % for i in range(0,(((n+1)*n)/2) - 1):
d_e[${i}], \\
        % endfor
d_e[${(((n+1)*n)/2) - 1}])
        % for i in range(0, len(y)):
            mu${i}[i] <- ${mmult2(context['x' + str(i+1)], 'beta', 'i', lenbeta)}\\
% for j in range(0, numlevs):
 + u${j}_${i}[${context['C' + str(j + 1)]}[i]]\\
% endfor
<% lenbeta += len(context['x' + str(i+1)]) %>
        % endfor
        % for i in range(0,n):
        miss${y[i]}[i] ~ dflat()
        % endfor
    % endif
        }

% for j in range(0, numlevs):
    for (i${j + 1} in 1:length(u${j}_0)) {
        %if len(y) == 1 :
           u${j}_0[i] ~ dnorm(0, tau_u)
    % else:
        dummy_u${j}[i${j + 1}] ~ dnormal${n}a( \\
        % for i in range(0,n):
u${j}_${i}[i${j + 1}], \\
        % endfor
        % for i in range(0,n):
0, \\
        % endfor
        % for i in range(0,(((n+1)*n)/2) - 1):
d_u${j+1}[${i}], \\
        % endfor
d_u${j+1}[${(((n+1)*n)/2) - 1}])
        dum_u${j}[i${j + 1}] ~ ddummy(dummy_u${j}[i${j + 1}])
        #above line to fix the DIC
        % for i in range(0,n):
        u${j}_${i}[i${j + 1}] ~ dflat()
        % endfor

    % endif
    }
% endfor
    # Priors

    % if n == 1:
    tau ~ dgamma(0.001, 0.001)
    sigma <- 1 / sqrt(tau)
    % for i in range(0, int(NumLevs)):
        tau_u${i} ~ dgamma(0.001, 0.001)
        sigma_u${i} <- 1 / sqrt(tau_u${i})
    % endfor
    % endif

    % for i in range(0, lenbeta):
    beta_${i} ~ dflat()
    % endfor
}
'''

    latex = r'''
<% lenbeta = 0 %>
<% numlevs = int(NumLevs) %>
\begin{aligned}
\left(
\begin{array}{l}
% for i in range(0, len(y)):
\mbox{${y[i]}}_i
% if i != len(y) -1:
\\ 
% endif
% endfor
\end{array}
\right) & \sim \mbox{N}
\left[ \left(
\begin{array}{l}
% for i in range(0, len(y)):
\mu_{${i}i}
% if i != len(y) -1:
\\ 
% endif
% endfor
\end{array}
\right), \Omega_e \right] \\ 
% for i in range(0, len(y)): 
\mu_{${i}i} & = ${mmulttex2(context['x' + str(i+1)], r'\beta', 'i', lenbeta)}
% for lev in range(0, numlevs):
 + u^{(${lev + 1})}_{${i},${context['C' + str(lev + 1)]}(i)}
% endfor
\\ 
<% lenbeta += len(context['x' + str(i+1)]) %>
% endfor
% for lev in range(0, numlevs):
\left(
\begin{array}{l}
% for i in range(0, len(y)):
u^{(${lev + 1})}_{${i},${context['C' + str(lev + 1)]}(i)}
% if i != len(y) -1:
\\ 
% endif
% endfor
\end{array}
\right) & \sim \mbox{N}
\left[ \left(
\begin{array}{l}
% for i in range(0, len(y)):
0
% if i != len(y) -1:
\\ 
% endif
% endfor
\end{array}
\right), \Omega^{(${lev + 1})}_{u} \right] \\ 
% endfor
% if priors == 'Wishart': 
\Omega_e & \sim \mbox{inverse Wishart}_{${len(y)}}[${v}*R_e,${v}],R_e=
\left[\begin{array}{
% for i in range(0, len(y)):
l
% endfor
}
<% lenR = 0%>
% for i in range(0, len(y)):
% for j in range(0, len(y)):
% if j <= i:
${R.name[lenR]}
<% lenR += 1 %>
% endif
% if j != len(y) - 1:
&
% else:
% if i != len(y) - 1:
\\ 
% endif
% endif
% endfor
% endfor
\end{array}
\right]
% endif
% if priors == 'Uniform':
\Omega_e & \propto 1
% endif
\\ 
% for lev in range(0, numlevs):
% if context['priors' + str(lev)] == 'Wishart':
\Omega^{(${lev + 1})}_u & \sim \mbox{inverse Wishart}_{${len(y)}}[${context['v' + str(lev)]}*R_{u${lev + 1}},${context['v' + str(lev)]}],R_{u${lev + 1}}=
\left[\begin{array}{
% for i in range(0, len(y)):
l
% endfor
}
<% lenR = 0%>
% for i in range(0, len(y)):
% for j in range(0, len(y)):
% if j <= i:
${context['R' + str(lev)][lenR]}
<% lenR += 1 %>
% endif
% if j != len(y) - 1:
&
% else:
% if i != len(y) - 1:
\\ 
% endif
% endif
% endfor
% endfor
\end{array}
\right]
% endif
% if context['priors' + str(lev)] == 'Uniform':
\Omega^{(${lev + 1})}_u & \propto 1
% endif
\\ 
% endfor
% for i in range(0, lenbeta):
\beta_{${i}} & \propto 1 \\ 
% endfor
\end{aligned}
'''

    customccode = '''
        <%!
    def mmult2(names, var, index,count):
        out = ""
        first = True
        for name in names:
            if first == False:
                out += ' + '
            else:
                first = False
            out += 'double(' + name + '[' + index + ']) * ' + var + '_' + str(count)
            count += 1
        return out
    def index(n, t):
        ind = ''
        for i in range(0, n):
            ind += '[i' + str(i + 1) + ']'
        if t != '':
            ind += '[' + t + ']'
        return ind
%>
    
    {
        bool fail = false;
<% n = len(y) %>\\
<% numlevs = int(NumLevs) %>\\

        // Note currently using a uniform prior for variance matrix
        SymMatrix sbe(${n});
        int vwe = length(${y[0]});

<% numbeta = 0 %>\\
<% lenbeta = 0 %>\\
        for (int i = 0; i < length(${y[0]}); i++) {
% for k in range(0, n):
<% lenbeta += len(context['x' + str(k + 1)]) %>\\
            double e${k} = double(miss${y[k]}[i]) - (\\
% for j in range(0, len(context['x' + str(k+1)])):
% if j != 0:
 + \\
% endif
double(${context['x' + str(k+1)][j]}[i]) * beta[${numbeta}] \\
<% numbeta += 1 %>\\
% endfor
)\\
% for i in range(0, numlevs):
 - double(u${i}_${k}[int(${context['C' + str(i + 1)]}[i])]) \\
% endfor
;

% endfor
% for i in range(0, n):
% for j in range(i, n):
            sbe(${i}, ${j}) += e${i} * e${j};
% endfor
% endfor
        }
% if priors == 'Uniform':
            if (runstate == 0) {
% for i in range(0, n):
                sbe(${i}, ${i}) += 0.0001;
% endfor
            }
% endif
% if priors == 'Wishart':
<%
import numpy
Rmat = numpy.empty([n, n])
count = 0
for i in range(0, n):
    for j in range(0, i + 1):
        Rmat[i, j] = float(R[count])
        Rmat[j, i] = Rmat[i, j]
        count += 1
%>

<% count = 0 %>
% for i in range(0, n):
% for j in range(i, n):
            sbe(${i}, ${j}) += ${str(Rmat[i, j] * float(v))};
% endfor
% endfor
% endif
            matrix_sym_invinplace(sbe);

% if priors == 'Uniform':
            vwe -= ${n + 1};
% endif
% if priors == 'Wishart':
            vwe += ${v};
% endif

            mat_d_e = dwishart(vwe, sbe, fail);
            mat_omega_e = matrix_sym_inverse(mat_d_e, fail);

% for k in range(0, numlevs):
        std::vector<double*> tmp_u${k};
% for i in range(0, n):
        tmp_u${k}.push_back(u${k}_${i});
% endfor
        RectMatrix mat_u${k}(tmp_u${k}, length(u${k}_0));

        SymMatrix sbu${k} = mat_u${k}.T() * mat_u${k};

% if context['priors' + str(k)] == 'Uniform':
        if (runstate == 0) {
% for i in range(0, n):
            sbu${k}(${i}, ${i}) += 0.0001;
% endfor
        }
% endif
% if context['priors' + str(k)] == 'Wishart':
<%
import numpy
Rmat = numpy.empty([n, n])
count = 0
for i in range(0, n):
    for j in range(0, i + 1):
        Rmat[i, j] = float(context['R' + str(k)][count])
        Rmat[j, i] = Rmat[i, j]
        count += 1
%>

<% count = 0 %>
% for i in range(0, n):
% for j in range(i, n):
        sbu${k}(${i}, ${j}) += ${str(Rmat[i, j] * float(context['v' + str(k)]))};
% endfor
% endfor
% endif
        matrix_sym_invinplace(sbu${k});
% if context['priors' + str(k)] == 'Uniform':
        int vwu${k} = length(u${k}_0) - ${n + 1};
% endif
% if context['priors' + str(k)] == 'Wishart':
        int vwu${k} = length(u${k}_0) + ${context['v' + str(k)]};
% endif

        mat_d_u${k+1} = dwishart(vwu${k}, sbu${k}, fail);
        mat_omega_u${k+1} = matrix_sym_inverse(mat_d_u${k+1}, fail);
% endfor

                // Step for beta
                std::vector<int> betaind(${lenbeta});
<% betacount = 0 %>
% for i in range(0, n):
% for j in range(0, len(context['x' +str(i+1)])):
                betaind[${betacount}] = ${i};
<% betacount += 1 %>
% endfor
% endfor
                std::vector<double*> tmp_x;
% for i in range(0, n):
% for j in range(0, len(context['x' + str(i+1)])):
                tmp_x.push_back((double*)${context['x' + str(i+1)][j]});
% endfor
% endfor
                RectMatrix mat_x(tmp_x, length(${y[0]}));

                std::vector<double*> tmp_ymiss;
% for i in range(0, n):
                tmp_ymiss.push_back(miss${y[i]});
% endfor
                RectMatrix mat_ymiss(tmp_ymiss, length(${y[0]}));

                SymMatrix db(${lenbeta});
                for (int i = 0; i < length(${y[0]}); i++) {
                    for (int j1 = 0; j1 < ${lenbeta}; j1++) {
                        for (int j2 = 0; j2 < j1+1; j2++) {
                            db(j1, j2) += mat_x(i, j1) * mat_d_e(betaind[j1], betaind[j2]) * mat_x(i, j2);
                        }
                    }
                }

                RectMatrix xtdy(${lenbeta}, 1);
                for (int i = 0; i < length(${y[0]}); i++) {
                    for (int j1 = 0; j1 < ${lenbeta}; j1++) {
                        for (int j2 = 0; j2 < ${n}; j2++) {
                            double tmp = 0.0;
% for j in range(0, numlevs):
                            tmp += mat_u${j}(${context['C' + str(j + 1)]}[i], j2);
% endfor
                            xtdy(j1, 0) += mat_x(i, j1) * mat_d_e(betaind[j1], j2) * (mat_ymiss(i, j2) - tmp);
                        }
                    }
                }

                fail = false;
                // int err = matrix_sym_invinplace(db);
                // RectMatrix betahat = db * xtdy;
                // mat_beta = dmultnormal(betahat, db, fail);

                RectMatrix betahat = matrix_solve(db, xtdy);
                mat_beta = dmultnormalinv(betahat, db, fail);

                std::vector<int> betamap(${n});
                int pos = 0;
% for i in range(0, n):
                betamap[${i}] = pos;
                pos += mat_x${i + 1}.cols();
% endfor

% for j in range(0, numlevs):
% for i in range(0, n):
                for (int j = 0; j < mat_u${j}_${i}.rows(); j++) {
                    std::vector<double> sumx(mat_x.cols());
                    std::vector<double> sumy(mat_ymiss.cols());
% for p in range(0, numlevs):
                    std::vector<double> sumu${p}(${n});
% endfor

                    for (int id_k = 0; id_k < idmap_${context['C' + str(j + 1)]}[j].size(); id_k++) {
                        int k = idmap_${context['C' + str(j + 1)]}[j][id_k];
                        for (int m = 0; m < mat_x.cols(); m++) {
                            sumx[m] += mat_x(k, m);
                        }
                        for (int m = 0; m < mat_ymiss.cols(); m++) {
                            sumy[m] += mat_ymiss(k, m);
                        }
% for p in range(0, numlevs):
% for m in range(0, n):
                        sumu${p}[${m}] += mat_u${p}_${m}(${context['C' + str(p + 1)]}[k], 0);
% endfor
% endfor
                    }

                    double tmp = 0.0;
                    for (int k = 0; k < mat_x${i + 1}.cols(); k++) {
                        tmp += mat_beta(betamap[${i}] + k, 0) * sumx[betamap[${i}] + k];
                    }
                    tmp -= sumy[${i}];
% for p in range(0, numlevs):
% if p != j:
                    tmp += sumu${p}[${i}];
% endif
% endfor
                    tmp *= mat_d_e(${i}, ${i});


                    {
                        double tmp2 = 0.0;
% for m in range(0, n):
                        tmp2 = 0.0;
% if i != m:
                        for (int k = 0; k < mat_x${m + 1}.cols(); k++) {
                            tmp2 += mat_beta(betamap[${m}] + k, 0) * sumx[betamap[${m}] + k];
                        }
                        tmp2 -= sumy[${m}];
% for p in range(0, numlevs):
                        tmp2 += sumu${p}[${m}];
% endfor
                        tmp2 *= mat_d_e(${i}, ${m});
% endif
                        tmp += tmp2;
% endfor
                    }


% for m in range(0, n):
% if i != m:
                    tmp += mat_d_u${j + 1}(${i},${m}) * mat_u${j}_${m}(j, 0);
% endif
% endfor


                    tmp /= (mat_d_u${j + 1}(${i}, ${i}) + mat_d_e(${i}, ${i}) * idmap_${context['C' + str(j + 1)]}[j].size());

                    mat_u${j}_${i}(j, 0) = dnorm(-tmp, mat_d_u${j + 1}(${i}, ${i}) + mat_d_e(${i}, ${i}) * idmap_${context['C' + str(j + 1)]}[j].size());
                }
% endfor
% endfor



% for i in range(0, n):
                for (int j = 0; j < mat_ymiss.rows(); j++) {
                    if (mat_y(j, ${i}) <= -9.999e29) {
                        double tmp = 0.0;
% for j in range(0, numlevs):
                        tmp += mat_u${j}_${i}(${context['C' + str(j + 1)]}[j], 0);
% endfor
                        for (int k = 0; k < mat_x${i + 1}.cols(); k++) {
                            tmp += mat_beta(betamap[${i}] + k, 0) * mat_x${i + 1}(j, k);
                        }
                        for (int k = 0; k < mat_ymiss.cols(); k++) {
                            if (k != ${i}) {
                                tmp -= mat_d_e(${i}, k) * mat_ymiss(j, k) * (1.0 / mat_d_e(${i}, ${i}));
                            }
                        }
% for m in range(0, n):
% if i != m:
% for p in range(0, numlevs):
                        tmp += mat_d_e(${i}, ${m}) * mat_u${p}_${m}(${context['C' + str(j + 1)]}[j], 0) * (1.0 / mat_d_e(${i}, ${i}));
% endfor
                        for (int k = 0; k < mat_x${m + 1}.cols(); k++) {
                            tmp += mat_d_e(${i}, ${m}) * (1 / mat_d_e(${i}, ${i})) * mat_beta(betamap[${m}] + k, 0) * mat_x${m + 1}(j, k);
                        }
% endif
% endfor
                        mat_ymiss(j, ${i}) = dnorm(tmp, mat_d_e(${i}, ${i}));
                    }
                }
% endfor
         }
'''

    preparedata = '''
mydata = data['datafile']

for i in range(0, len(y)):
    context['miss'+str(y[i])].ncols = -1*len(mydata.variables[y[i]]['data'])
'''
